package com.caimei.config;

import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.util.HttpRequest;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 微信工具类
 *
 * @author LG
 * @create 2017-09-21
 **/
@Slf4j
@Component
public class WxConfig {
    public static String CrmAppId;

    @Value("${wx.crmAppId}")
    public void setCrmAppId(String crmAppId) {
        CrmAppId = crmAppId;
    }

    public static String CrmAppSecret;

    @Value("${wx.crmAppSecret}")
    public void setCrmAppSecret(String crmAppSecret) {
        CrmAppSecret = crmAppSecret;
    }

    public static String AppId;

    @Value("${wx.AppId}")
    public void setAppId(String appId) {
        AppId = appId;
    }

    public static String AppSecret;

    @Value("${wx.AppSecret}")
    public void setAppSecret(String appSecret) {
        AppSecret = appSecret;
    }

    /**
     * 网页授权登录,通过code获取access_token
     *
     * @param code wxcode
     * @return
     * @throws Exception
     */
    public static Map<String, Object> getAccessTokenMap(String code) throws Exception {
        String link = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=SECRET&code=CODE&grant_type=authorization_code";
        //微信公众号
        link = link.replace("APPID", CrmAppId);
        link = link.replace("SECRET", CrmAppSecret);
        //填写第一步获取的code参数
        link = link.replace("CODE", code);
        String result = HttpRequest.sendGet(link);
        log.info(result);
        Map<String, Object> map = JSONObject.parseObject(result, Map.class);
        return map;
    }

    /**
     * 小程序授权,获取openId
     *
     * @param code
     * @return
     */
    public static String getWxMallOpenId(String code) throws Exception {
        String requestUrl = "https://api.weixin.qq.com/sns/jscode2session";
        Map<String, String> requestUrlParam = new HashMap<>(4);
        //小程序appId
        requestUrlParam.put("appid", AppId);
        //小程序appSecret
        requestUrlParam.put("secret", AppSecret);
        //小程序端返回的code
        requestUrlParam.put("js_code", code);
        //默认参数
        requestUrlParam.put("grant_type", "authorization_code");
        //发送post请求读取调用微信接口获取openid用户唯一标识
        return HttpRequest.sendPost(requestUrl, requestUrlParam);
    }
}
